#include "parser.h"

#include <io.h>
#include <fcntl.h>
#include <stdio.h>

Parser::err_t ParserHex::open(const char*filename, bool write) {
 if (write) return ERR_RDONLY;
#ifdef UNICODE	// filename may contain non-ASCII characters, so use wide-char open function
 int len=MultiByteToWideChar(CP_UTF8,0,filename,-1,0,0);
 wchar_t*n=new wchar_t[len];
 MultiByteToWideChar(CP_UTF8,0,filename,-1,n,len);
 int fd=_wopen(n,O_RDONLY);
 delete[]n;
#elif defined(_WIN64)
 int fd = old_msvcrt::_open(filename, O_RDONLY);
#else
 int fd = _open(filename, O_RDONLY);
#endif
 if (fd<0) return ERR_SYSTEM;
 char mark;
 while (_read(fd,&mark,1)) {
  unsigned int c, i;
  uint32_t base = 0;
  unsigned int last_address = 0x0;
		/* read in the file */
  char buffer[9];
  unsigned int reclen, address, type;
  uint8_t *record = NULL;

  if (mark == '\n' || mark == '\r') continue;
  if (mark != ':') return ERR_INVALID_FILE;
		/* get the reclen, address, and type */
  buffer[8] = 0;
  if (_read(fd, &buffer, 8) != 8) return ERR_INVALID_FILE;
  if (sscanf(buffer, "%2x%4x%2x", &reclen, &address, &type) != 3) {
   _close(fd);
   return ERR_INVALID_FILE;
  }
			/* setup the checksum */
  uint8_t checksum = reclen + ((address & 0xFF00) >> 8) + ((address & 0x00FF) >> 0) + type;
  switch(type) {		/* data record */
   case 0:
    c = address - last_address;
    data = (uint8_t*)realloc(data, data_len + c + reclen);
					/* if there is a gap, set it to 0xff and increment the length */
    if (c > 0) {
     memset(&data[data_len], 0xff, c);
     data_len += c;
    }
    last_address = address + reclen;
    record = &data[data_len];
    data_len += reclen;
    break;

   case 2:	/* extended segment address record */
    base = 0;
    break;

   case 4:	/* extended linear address record */
    base = 0;
    break;
  }
  buffer[2] = 0;
  for(i = 0; i < reclen; ++i) {
   if (_read(fd, &buffer, 2) != 2 || sscanf(buffer, "%2x", &c) != 1) {
   _close(fd);
   return ERR_INVALID_FILE;
  }
				/* add the byte to the checksum */
  checksum += c;
  switch(type) {
   case 0: if (record) record[i] = c;
   else return ERR_INVALID_FILE;
   case 2:
   case 4: base = (base << 8) | c; break;
  }
 }
			/* read, scan, and verify the checksum */
 if (_read(fd, &buffer, 2 ) != 2
  || sscanf(buffer, "%2x", &c) != 1
  || (uint8_t)(checksum + c)) {
	_close(fd);
	return ERR_INVALID_FILE;
 }
 switch(type) {		/* EOF */
  case 1: _close(fd); return OK;
				/* address record */
  case 4:	base = base << 12;
					/* fall-through */
  case 2: base = base << 4;
					/* Reset last_address since our base changed */
  last_address = 0;

					/* Only assign the program's base address once, and only
					 * do so if we haven't seen any data records yet.
					 * If there are any data records before address records,
					 * the program's base address must be zero.
					 */
  if (!base && !data_len) {this->base = base;break;
 }

				/* we cant cope with files out of order */
 if (base < this->base) {
  _close(fd);
  return ERR_INVALID_FILE;
 }

					/* if there is a gap, enlarge and fill with 0xff */
 {unsigned len = base - this->base;
  if (len > data_len) {
  data = (uint8_t*)realloc(this->data, len);
  memset(&this->data[data_len], 0xff, len - data_len);
  data_len = len;
 }}
			break;
		}
	}
 _close(fd);
 return OK;
}

Parser::err_t ParserHex::read(void *dat, unsigned int&len) {
 unsigned left = data_len - offset;
 unsigned get  = left > len ? len : left;
 memcpy(dat, data+offset, get);
 offset += get;
 len = get;
 return OK;
}
