#pragma once
#include "../port.h"

struct Parser {
 enum err_t {
  OK,
  ERR_SYSTEM,
  ERR_INVALID_FILE,
  ERR_WRONLY,
  ERR_RDONLY
 };
 const char*name;
 Parser(const char*n):name(n) {}
 virtual ~Parser() {};					// close and free the parser
 virtual err_t open(const char*filename, bool write)=0;	// open the file for read|write
 virtual unsigned size() const=0;				// get the total data size
 virtual err_t read(void *data, unsigned&len)=0;	// read a block of data
 virtual err_t write(const void *data, unsigned len)=0;	// write a block of data
 static const char*errstr(err_t);
};

struct ParserHex:public Parser {
 ParserHex():Parser("Intel HEX"),data_len(0),offset(0),data(0),base(0) {}
 virtual ~ParserHex() {delete[] data;}				// close and free the parser
 virtual err_t open(const char*filename, bool write) override;	// open the file for read|write
 virtual unsigned size() const override {return data_len;}	// get the total data size
 virtual err_t read(void *data, unsigned&len) override;		// read a block of data
 virtual err_t write(const void *data, unsigned len) override {return ERR_RDONLY;}
private:
 unsigned	data_len, offset;
 uint8_t	*data;
 uint32_t	base;
};

struct ParserBin:public Parser {
 ParserBin():Parser("Raw BINARY"),fd(-1),filesize(0) {}
 virtual ~ParserBin();						// close and free the parser
 virtual err_t open(const char*filename, bool write) override;	// open the file for read|write
 virtual unsigned size() const override {return filesize;}	// get the total data size
 virtual err_t read(void *data, unsigned&len) override;		// read a block of data
 virtual err_t write(const void *data, unsigned len) override;	// write a block of data
private:
 int	fd;
 bool	wr;
 unsigned filesize;
};

#ifdef _WIN64
namespace old_msvcrt{
EXTERN_C _CRTIMP int _cdecl _open(const char*,int,...);
}
#endif
